@@ -24,6 +24,7 @@
 #include <limits.h>
 #include <ctype.h>
 
+#include <gconf/gconf-client.h>
 #include <panel-applet-gconf.h>
 #include <egg-screen-help.h>
 #include "geyes.h"
@@ -134,6 +135,25 @@ destroy_theme (EyesApplet *eyes_applet)
         g_free (eyes_applet->theme_name);
 }
 
+static gboolean
+key_writable (PanelApplet *applet, const char *key)
+{
+	gboolean writable;
+	char *fullkey;
+	static GConfClient *client = NULL;
+
+	if (client == NULL)
+		client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+	g_free (fullkey);
+
+	return writable;
+}
+
 static void
 theme_selected_cb (GtkTreeSelection *selection, gpointer data)
 {
@@ -219,7 +239,11 @@ properties_cb (BonoboUIComponent *uic,
         DIR *dfd;
         struct dirent *dp;
         int i;
+#ifdef PATH_MAX
         gchar filename [PATH_MAX];
+#else
+	gchar *filename;
+#endif
         gchar *title;
      
 	if (eyes_applet->prop_box.pbox) {
@@ -240,13 +264,14 @@ properties_cb (BonoboUIComponent *uic,
         gtk_window_set_resizable (GTK_WINDOW (pbox), FALSE);
         gtk_dialog_set_default_response(GTK_DIALOG (pbox), GTK_RESPONSE_CLOSE);
         gtk_dialog_set_has_separator (GTK_DIALOG (pbox), FALSE);
+        gtk_container_set_border_width (GTK_CONTAINER (pbox), 5);
 
         g_signal_connect (pbox, "response",
 			  G_CALLBACK (presponse_cb),
 			  eyes_applet);
 	
 	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_widget_show (vbox);
 	
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pbox)->vbox), vbox,
@@ -297,10 +322,15 @@ properties_cb (BonoboUIComponent *uic,
         gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
                                                            
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-        g_signal_connect (selection, "changed",
-        		  G_CALLBACK (theme_selected_cb),
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (theme_selected_cb),
 			  eyes_applet);
-        
+
+	if ( ! key_writable (eyes_applet->applet, "theme_path")) {
+		gtk_widget_set_sensitive (tree, FALSE);
+		gtk_widget_set_sensitive (label, FALSE);
+	}
+
         for (i = 0; i < NUM_THEME_DIRECTORIES; i++) {
                 if ((dfd = opendir (theme_directories[i])) != NULL) {
                         while ((dp = readdir (dfd)) != NULL) {
@@ -308,9 +338,13 @@ properties_cb (BonoboUIComponent *uic,
                                         gchar *elems[2] = {NULL, NULL };
                                         gchar *theme_dir;
 					elems[0] = filename;
+#ifdef PATH_MAX
                                         strcpy (filename, 
                                                 theme_directories[i]);
                                         strcat (filename, dp->d_name);
+#else
+					asprintf (&filename, theme_directories[i], dp->d_name);
+#endif
                                         gtk_list_store_insert (model, &iter, 0);
                                         gtk_list_store_set (model, &iter, 0, &filename, -1);
                                         theme_dir = g_strdup_printf ("%s/", filename);
@@ -330,6 +364,9 @@ properties_cb (BonoboUIComponent *uic,
                         closedir (dfd);
                 }
         }
+#ifndef PATH_MAX
+	g_free (filename);
+#endif
         
         gtk_box_pack_start (GTK_BOX (control_vbox), tree, TRUE, TRUE, 0);
         
