@@ -46,7 +46,8 @@
 #include <libgnome/libgnome.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-window-icon.h>
-#include <panel-applet.h>
+#include <gconf/gconf-client.h>
+#include <panel-applet-gconf.h>
 #include <egg-screen-exec.h>
 #include <egg-screen-help.h>
 
@@ -161,7 +162,11 @@ static void mixer_popup_hide    (MixerData *data, gboolean revert);
 static void mixer_start_gmix_cb (BonoboUIComponent *uic,
 				 MixerData         *data,
 				 const gchar       *verbname);
-
+static void mixer_ui_component_event (BonoboUIComponent *comp,
+			  const gchar                  *path,
+			  Bonobo_UIComponent_EventType  type,
+			  const gchar                  *state_string,
+			  MixerData                    *data);
 void add_atk_namedesc (GtkWidget *widget, const gchar *name, const gchar *desc);
 
 static gint mixerfd = -1;
@@ -184,6 +189,24 @@ long pv_buf[MAX_PV_BUF] = {
 };
 #endif
 
+static gboolean
+key_writable (PanelApplet *applet, const char *key)
+{
+	gboolean writable;
+	char *fullkey;
+	static GConfClient *client = NULL;
+	if (client == NULL)
+		client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+	g_free (fullkey);
+
+	return writable;
+}
+
 #ifdef OSS_API
 static void
 get_channels (MixerData *data)
@@ -211,8 +234,9 @@ get_channels (MixerData *data)
 	if (!valid) {
 		ChannelData *cdata = data->channels->data;
 		data->mixerchannel = cdata->channel;
-		panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "channel", 
-							  data->mixerchannel, NULL);
+		if (key_writable (PANEL_APPLET (data->applet), "channel"))
+			panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "channel", 
+						    data->mixerchannel, NULL);
 	}
 }
 #endif
@@ -377,7 +401,7 @@ mixer_timeout_cb (MixerData *data)
 	BonoboUIComponent *component;
 	gint               vol;
 	gboolean state;
-
+	
 	if (!device_present) {
 		data->mute = TRUE;
 		mixer_update_image (data);
@@ -472,6 +496,7 @@ mixer_timeout_cb (MixerData *data)
 		mixer_update_slider (data);
 	}
 	
+	
 	return TRUE;
 }
 
@@ -551,6 +576,14 @@ mixer_update_image (MixerData *data)
 	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), scaled);
 	g_object_unref (scaled);	
 	g_object_unref (pixbuf);
+	
+	if (key_writable (PANEL_APPLET (data->applet), "vol"))
+		panel_applet_gconf_set_int (PANEL_APPLET (data->applet), "vol", data->vol,
+					    NULL);
+	if (key_writable (PANEL_APPLET (data->applet), "mute"))
+		panel_applet_gconf_set_bool (PANEL_APPLET (data->applet), "mute", data->mute,
+					     NULL);
+			
 }
 
 static gboolean
@@ -561,7 +594,7 @@ scale_key_press_event_cb (GtkWidget *widget, GdkEventKey *event, MixerData *data
 		/* Revert. */
 		mixer_popup_hide (data, TRUE);
 		return TRUE;
-
+	
 	case GDK_KP_Enter:
 	case GDK_ISO_Enter:
 	case GDK_3270_Enter:
@@ -649,7 +682,22 @@ applet_key_press_event_cb (GtkWidget *widget, GdkEventKey *event, MixerData *dat
 		/* Revert. */
 		mixer_popup_hide (data, TRUE);
 		return TRUE;
-
+	case GDK_m:
+		if (event->state == GDK_CONTROL_MASK) {
+			Bonobo_UIComponent_EventType  type;
+			if (data->mute)
+				mixer_ui_component_event (NULL, "Mute", type, "0", data);
+			else
+				mixer_ui_component_event (NULL, "Mute", type, "1", data);
+			return TRUE;
+		}
+		break;
+	case GDK_o:
+		if (event->state == GDK_CONTROL_MASK) {
+			mixer_start_gmix_cb (NULL, data, NULL);
+			return TRUE;
+		}
+		break;
 	case GDK_KP_Enter:
 	case GDK_ISO_Enter:
 	case GDK_3270_Enter:
@@ -1241,7 +1289,12 @@ mixer_pref_cb (BonoboUIComponent *uic,
 		}
         	list = g_list_next (list);
         }
-        
+
+	if ( ! key_writable (PANEL_APPLET (data->applet), "channel")) {
+		gtk_widget_set_sensitive (tree, FALSE);
+		gtk_widget_set_sensitive (label, FALSE);
+	}
+
         g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree))), 
 			  	  "changed",
 			  	   G_CALLBACK (cb_row_selected), data);
@@ -1262,7 +1315,7 @@ mixer_ui_component_event (BonoboUIComponent            *comp,
 			  MixerData                    *data)
 {
 	gboolean state;
-	
+
 	if (!strcmp (path, "Mute")) {
 		if (!strcmp (state_string, "1")) {
 			state = TRUE;
@@ -1318,15 +1371,71 @@ mixer_ui_component_event (BonoboUIComponent            *comp,
 }
 
 static void
-get_prefs (MixerData *data)
-{
+get_channel (MixerData *data)
+{	
+#ifdef OSS_API
 	PanelApplet *applet = PANEL_APPLET (data->applet);
 	gint num;
-#ifdef OSS_API
+	
 	num = panel_applet_gconf_get_int (applet, "channel", NULL);
 	num = CLAMP (num, 0,  SOUND_MIXER_NRDEVICES-1);	
 	data->mixerchannel = num;
 #endif
+
+}
+
+static void
+get_prefs (MixerData *data)
+{
+	PanelApplet *applet = PANEL_APPLET (data->applet);
+	BonoboUIComponent *component;
+	gboolean mute;
+	
+	data->vol = panel_applet_gconf_get_int (applet, "vol", NULL);
+	mute = panel_applet_gconf_get_bool (applet, "mute", NULL);
+	
+	if (data->vol < 0)
+		return;
+		
+	data->mute = mute;
+	if (data->mute) {
+#ifdef SUN_API
+		set_mute_status (TRUE);
+#else
+		setMixer (0, data);
+#endif
+		mixer_update_image (data);
+		
+				
+		gtk_tooltips_set_tip (data->tooltips,
+					      data->applet,
+					      _(access_name_mute),
+					      NULL); 
+		if (gail_loaded) {
+				add_atk_namedesc (data->applet,
+						  _(access_name_mute),
+						  NULL);
+		}
+	}
+	else {
+#ifdef SUN_API
+		set_mute_status (FALSE);
+#else
+		setMixer (data->vol, data);
+#endif
+		mixer_update_image (data);
+			
+		
+		gtk_tooltips_set_tip (data->tooltips,
+				      data->applet,
+				      _(access_name),
+				      NULL); 
+		if (gail_loaded) {
+			add_atk_namedesc (data->applet,
+					  _(access_name),
+					  NULL);
+		}
+	}
 }
 
 typedef struct
@@ -1419,8 +1528,9 @@ mixer_applet_create (PanelApplet *applet)
 
 	data = g_new0 (MixerData, 1);
 	data->applet = GTK_WIDGET (applet);
+	panel_applet_add_preferences (applet, "/schemas/apps/mixer_applet/prefs", NULL);
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
-	get_prefs (data);
+	get_channel (data);
 
 #ifdef OSS_API
 	/* /dev/sound/mixer for devfs */
@@ -1510,7 +1620,10 @@ mixer_applet_create (PanelApplet *applet)
 			  data);
 
 	/* Get the initial volume. */
-	data->vol = readMixer (data);
+	get_prefs (data);
+	
+	if (!data->mute)
+		data->vol = readMixer (data);
 
 	/* Install timeout handler, that keeps the applet up-to-date if the
 	 * volume is changed somewhere else.
@@ -1519,8 +1632,6 @@ mixer_applet_create (PanelApplet *applet)
 					 (GSourceFunc) mixer_timeout_cb,
 					 data);
 
-	data->mute = FALSE;
-
 	g_signal_connect (applet,
 			  "destroy",
 			  (GCallback) destroy_mixer_cb,
@@ -1553,6 +1664,18 @@ mixer_applet_create (PanelApplet *applet)
 			  "ui-event",
 			  (GCallback) mixer_ui_component_event,
 			  data);
+	if (data->mute)
+		bonobo_ui_component_set_prop (component,
+						      "/commands/Mute",
+						      "state",
+						      "1",
+						      NULL);
+	else
+		bonobo_ui_component_set_prop (component,
+						      "/commands/Mute",
+						      "state",
+						      "0",
+						      NULL);
 
 	if (run_mixer_cmd == NULL)
 		bonobo_ui_component_rm (component, "/popups/popup/RunMixer", NULL);
