@@ -64,9 +64,9 @@ StickyNote * stickynote_new()
 						      gconf_client_get_int(stickynotes->gconf, GCONF_PATH "/defaults/height", NULL));
 
 	/* Set the button images */
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "close_img")), STICKYNOTES_STOCK_CLOSE, STICKYNOTES_ICON_SIZE);
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "resize_se_img")), STICKYNOTES_STOCK_RESIZE_SE, STICKYNOTES_ICON_SIZE);
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "resize_sw_img")), STICKYNOTES_STOCK_RESIZE_SW, STICKYNOTES_ICON_SIZE);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "close_img")), STICKYNOTES_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "resize_se_img")), STICKYNOTES_STOCK_RESIZE_SE, GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "resize_sw_img")), STICKYNOTES_STOCK_RESIZE_SW, GTK_ICON_SIZE_MENU);
 	
 	/* Customize the title and colors, hide and unlock */
 	stickynote_set_title(note, NULL);
@@ -84,6 +84,7 @@ StickyNote * stickynote_new()
 
 	/* Connect a properties dialog to the note */
 	gtk_window_set_transient_for(GTK_WINDOW(note->w_properties), GTK_WINDOW(note->w_window));
+	gtk_dialog_set_default_response(GTK_DIALOG(note->w_properties), GTK_RESPONSE_CLOSE);
 	
 	/* Connect signals to the sticky note */
 
@@ -112,7 +113,6 @@ StickyNote * stickynote_new()
 	g_signal_connect(G_OBJECT(note->w_font), "font_set", G_CALLBACK(properties_font_cb), note);
 	g_signal_connect_swapped(G_OBJECT(note->w_def_font), "toggled", G_CALLBACK(properties_apply_font_cb), note);
 	g_signal_connect(G_OBJECT(note->w_entry), "activate", G_CALLBACK(properties_activate_cb), note);
-	g_signal_connect(G_OBJECT(note->w_properties), "response", G_CALLBACK(properties_response_cb), note);
 	
 	return note;
 }
@@ -151,6 +151,7 @@ void stickynote_change_properties(StickyNote *note)
 		gnome_font_picker_set_font_name(GNOME_FONT_PICKER(note->w_font), note->font);
 
 	gtk_dialog_run(GTK_DIALOG(note->w_properties));
+	gtk_widget_hide(note->w_properties);
 
 	stickynotes_save();
 }
@@ -307,15 +308,19 @@ void stickynote_set_locked(StickyNote *note, gboolean locked)
 
 	/* Show appropriate icon and tooltip */
 	if (locked) {
-		gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "lock_img")), STICKYNOTES_STOCK_LOCK, STICKYNOTES_ICON_SIZE);
+		gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "lock_img")),
+					 STICKYNOTES_STOCK_LOCKED, GTK_ICON_SIZE_MENU);
 		gtk_tooltips_set_tip(stickynotes->tooltips, note->w_lock, _("Locked note"), NULL);
 	}
 	else {
-		gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "lock_img")), STICKYNOTES_STOCK_UNLOCK, STICKYNOTES_ICON_SIZE);
+		gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(note->window, "lock_img")),
+					 STICKYNOTES_STOCK_UNLOCKED, GTK_ICON_SIZE_MENU);
 		gtk_tooltips_set_tip(stickynotes->tooltips, note->w_lock, _("Unlocked note"), NULL);
 	}
 
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(note->menu, "popup_toggle_lock")), locked);
+	
+	stickynotes_applet_update_menus();
 }
 
 /* Show/Hide a sticky note */
@@ -350,7 +355,10 @@ void stickynotes_add()
 
 	/* Show all sticky notes */
 	stickynote_set_visible(note, TRUE);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/visible", TRUE, NULL);
+
+	/* Only set the visible flag if we can actually write it */
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/visible", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/visible", TRUE, NULL);
 }
 
 /* Remove a sticky note with confirmation, if needed */
@@ -361,9 +369,9 @@ void stickynotes_remove(StickyNote *note)
 
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(note->w_window));
 
-	if (stickynote_get_empty(note) ||
-	    !gconf_client_get_bool(stickynotes->gconf, GCONF_PATH "/settings/confirm_deletion", NULL) ||
-	    gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
+	if (stickynote_get_empty(note)
+	    || !gconf_client_get_bool(stickynotes->gconf, GCONF_PATH "/settings/confirm_deletion", NULL)
+	    || gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
 		stickynote_free(note);
 
 		/* Remove the note from the linked-list of all notes */
