@@ -44,18 +44,10 @@ gboolean applet_button_cb(GtkWidget *widget, GdkEventButton *event, StickyNotesA
 gboolean applet_key_cb(GtkWidget *widget, GdkEventKey *event, StickyNotesApplet *applet)
 {
 	if (event->type == GDK_KEY_PRESS && event->keyval == GDK_Return)
-		applet->pressed = TRUE;
-	
-	else if (event->type == GDK_KEY_RELEASE && event->keyval == GDK_Return) {
 		stickynotes_applet_do_default_action();	
-		applet->pressed = FALSE;
-	}
-	
 	else
 		return FALSE;
 	
-	stickynotes_applet_update_icon(applet);
-	
 	return TRUE;
 }
 
@@ -146,13 +138,15 @@ void menu_destroy_all_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, cons
 /* Menu Callback : Show/Hide sticky notes */
 void menu_toggle_show_cb(BonoboUIComponent *uic, const gchar *path, Bonobo_UIComponent_EventType type, const gchar *state, StickyNotesApplet *applet)
 {
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/visible", strcmp(state, "0") != 0, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/visible", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/visible", strcmp(state, "0") != 0, NULL);
 }
 
 /* Menu Callback: Lock/Unlock sticky notes */
 void menu_toggle_lock_cb(BonoboUIComponent *uic, const gchar *path, Bonobo_UIComponent_EventType type, const gchar *state, StickyNotesApplet *applet)
 {
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/locked", strcmp(state, "0") != 0, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/locked", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/locked", strcmp(state, "0") != 0, NULL);
 }
 
 /* Menu Callback : Configure preferences */
@@ -171,7 +165,28 @@ void menu_help_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, const gchar
 /* Menu Callback : Display About window */
 void menu_about_cb(BonoboUIComponent *uic, StickyNotesApplet *applet, const gchar *verbname)
 {
-	gtk_window_present(GTK_WINDOW(stickynotes->w_about));
+	static const gchar *authors[] = {
+		"Loban A Rahman <loban@earthling.net>",
+		NULL
+	};
+
+	static const gchar *documenters[] = {
+		"Loban A Rahman <loban@earthling.net>",
+		NULL
+	};
+
+	const gchar *translator_credits = _("translator_credits");
+
+	GtkWidget *w_about = gnome_about_new(_("Sticky Notes"),
+					     VERSION,
+					     _("(c) 2002-2003 Loban A Rahman"),
+					     _("Sticky Notes for the Gnome Desktop Environment"),
+					     authors,
+					     documenters,
+					     strcmp(translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+					     stickynotes->icon_normal);
+
+	gtk_widget_show(w_about);
 }
 
 /* Preferences Callback : Save. */
@@ -185,13 +200,20 @@ void preferences_save_cb(gpointer data)
 	gboolean sticky = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stickynotes->w_prefs_sticky));
 	gboolean force_default = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stickynotes->w_prefs_force));
 
-	gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/width", width, NULL);
-	gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/height", height, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_color", sys_color, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_font", sys_font, NULL);
-	gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/settings/click_behavior", click_behavior, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/sticky", sticky, NULL);
-	gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/force_default", force_default, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/defaults/width", NULL))
+		gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/width", width, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/defaults/height", NULL))
+		gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/defaults/height", height, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/use_system_color", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_color", sys_color, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/use_system_font", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/use_system_font", sys_font, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/click_behavior", NULL))
+		gconf_client_set_int(stickynotes->gconf, GCONF_PATH "/settings/click_behavior", click_behavior, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/sticky", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/sticky", sticky, NULL);
+	if (gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/force_default", NULL))
+		gconf_client_set_bool(stickynotes->gconf, GCONF_PATH "/settings/force_default", force_default, NULL);
 }
 
 /* Preferences Callback : Change color. */
@@ -276,14 +298,14 @@ void preferences_response_cb(GtkDialog *dialog, gint response, gpointer data)
 {
 	if (response == GTK_RESPONSE_HELP)
 		gnome_help_display("stickynotes_applet", "stickynotes-introduction", NULL);
-	
-	else /* if (response == GTK_RESPONSE_CLOSE  || response == GTK_RESPONSE_DELETE_EVENT || response == GTK_RESPONSE_NONE)*/
+	else
 		gtk_widget_hide(GTK_WIDGET(dialog));
 }
 
-/* About Callback : Response. */
-void about_response_cb(GtkDialog *dialog, gint response, gpointer data)
+/* Preferences Callback : Delete */
+gboolean preferences_delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-	if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_DELETE_EVENT || response == GTK_RESPONSE_NONE)
-		gtk_widget_hide(GTK_WIDGET(dialog));
+	gtk_widget_hide(widget);
+
+	return TRUE;
 }
